@@ -11,6 +11,7 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 import sys
+import re
 import shlex
 import os
 import os.path
@@ -22,6 +23,8 @@ import contextlib
 from botocore.compat import six
 #import botocore.compat
 
+from botocore.compat import OrderedDict
+
 # If you ever want to import from the vendored six. Add it here and then
 # import from awscli.compat. Also try to keep it in alphabetical order.
 # This may get large.
@@ -97,6 +100,7 @@ def ensure_text_type(s):
 
 
 if six.PY3:
+    import collections.abc as collections_abc
     import locale
     import urllib.parse as urlparse
 
@@ -146,6 +150,7 @@ if six.PY3:
 
 else:
     import codecs
+    import collections as collections_abc
     import locale
     import io
     import urlparse
@@ -342,68 +347,181 @@ def ignore_user_entered_signals():
             signal.signal(user_signal, actual_signals[sig])
 
 
-def _backport_which(cmd, mode=os.F_OK | os.X_OK, path=None):
-    """Given a command, mode, and a PATH string, return the path which
-    conforms to the given mode on the PATH, or None if there is no such
-    file.
-    `mode` defaults to os.F_OK | os.X_OK. `path` defaults to the result
-    of os.environ.get("PATH"), or can be overridden with a custom search
-    path.
-    """
-    # Check that a given file can be accessed with the correct mode.
-    # Additionally check that `file` is not a directory, as on Windows
-    # directories pass the os.access check.
-    def _access_check(fn, mode):
-        return (os.path.exists(fn) and os.access(fn, mode)
-                and not os.path.isdir(fn))
-
-    # If we're given a path with a directory part, look it up directly rather
-    # than referring to PATH directories. This includes checking relative to the
-    # current directory, e.g. ./script
-    if os.path.dirname(cmd):
-        if _access_check(cmd, mode):
-            return cmd
-        return None
-
-    if path is None:
-        path = os.environ.get("PATH", os.defpath)
-    if not path:
-        return None
-    path = path.split(os.pathsep)
-
-    if sys.platform == "win32":
-        # The current directory takes precedence on Windows.
-        if not os.curdir in path:
-            path.insert(0, os.curdir)
-
-        # PATHEXT is necessary to check on Windows.
-        pathext = os.environ.get("PATHEXT", "").split(os.pathsep)
-        # See if the given file matches any of the expected path extensions.
-        # This will allow us to short circuit when given "python.exe".
-        # If it does match, only test that one, otherwise we have to try
-        # others.
-        if any(cmd.lower().endswith(ext.lower()) for ext in pathext):
-            files = [cmd]
-        else:
-            files = [cmd + ext for ext in pathext]
-    else:
-        # On other platforms you don't have things like PATHEXT to tell you
-        # what file suffixes are executable, so just pass on cmd as-is.
-        files = [cmd]
-
-    seen = set()
-    for dir in path:
-        normdir = os.path.normcase(dir)
-        if not normdir in seen:
-            seen.add(normdir)
-            for thefile in files:
-                name = os.path.join(dir, thefile)
-                if _access_check(name, mode):
-                    return name
-    return None
-
-
+# linux_distribution is used by the CodeDeploy customization. Python 3.8
+# removed it from the stdlib, so it is vendored here in the case where the
+# import fails.
 try:
-    from shutil import which
+    from platform import linux_distribution
 except ImportError:
-    which = _backport_which
+    _UNIXCONFDIR = '/etc'
+    def _dist_try_harder(distname, version, id):
+
+        """ Tries some special tricks to get the distribution
+            information in case the default method fails.
+            Currently supports older SuSE Linux, Caldera OpenLinux and
+            Slackware Linux distributions.
+        """
+        if os.path.exists('/var/adm/inst-log/info'):
+            # SuSE Linux stores distribution information in that file
+            distname = 'SuSE'
+            with open('/var/adm/inst-log/info') as f:
+                for line in f:
+                    tv = line.split()
+                    if len(tv) == 2:
+                        tag, value = tv
+                    else:
+                        continue
+                    if tag == 'MIN_DIST_VERSION':
+                        version = value.strip()
+                    elif tag == 'DIST_IDENT':
+                        values = value.split('-')
+                        id = values[2]
+            return distname, version, id
+
+        if os.path.exists('/etc/.installed'):
+            # Caldera OpenLinux has some infos in that file (thanks to Colin Kong)
+            with open('/etc/.installed') as f:
+                for line in f:
+                    pkg = line.split('-')
+                    if len(pkg) >= 2 and pkg[0] == 'OpenLinux':
+                        # XXX does Caldera support non Intel platforms ? If yes,
+                        #     where can we find the needed id ?
+                        return 'OpenLinux', pkg[1], id
+
+        if os.path.isdir('/usr/lib/setup'):
+            # Check for slackware version tag file (thanks to Greg Andruk)
+            verfiles = os.listdir('/usr/lib/setup')
+            for n in range(len(verfiles)-1, -1, -1):
+                if verfiles[n][:14] != 'slack-version-':
+                    del verfiles[n]
+            if verfiles:
+                verfiles.sort()
+                distname = 'slackware'
+                version = verfiles[-1][14:]
+                return distname, version, id
+
+        return distname, version, id
+
+    _release_filename = re.compile(r'(\w+)[-_](release|version)', re.ASCII)
+    _lsb_release_version = re.compile(r'(.+)'
+                                      r' release '
+                                      r'([\d.]+)'
+                                      r'[^(]*(?:\((.+)\))?', re.ASCII)
+    _release_version = re.compile(r'([^0-9]+)'
+                                  r'(?: release )?'
+                                  r'([\d.]+)'
+                                  r'[^(]*(?:\((.+)\))?', re.ASCII)
+
+    # See also http://www.novell.com/coolsolutions/feature/11251.html
+    # and http://linuxmafia.com/faq/Admin/release-files.html
+    # and http://data.linux-ntfs.org/rpm/whichrpm
+    # and http://www.die.net/doc/linux/man/man1/lsb_release.1.html
+
+    _supported_dists = (
+        'SuSE', 'debian', 'fedora', 'redhat', 'centos',
+        'mandrake', 'mandriva', 'rocks', 'slackware', 'yellowdog', 'gentoo',
+        'UnitedLinux', 'turbolinux', 'arch', 'mageia')
+
+    def _parse_release_file(firstline):
+
+        # Default to empty 'version' and 'id' strings.  Both defaults are used
+        # when 'firstline' is empty.  'id' defaults to empty when an id can not
+        # be deduced.
+        version = ''
+        id = ''
+
+        # Parse the first line
+        m = _lsb_release_version.match(firstline)
+        if m is not None:
+            # LSB format: "distro release x.x (codename)"
+            return tuple(m.groups())
+
+        # Pre-LSB format: "distro x.x (codename)"
+        m = _release_version.match(firstline)
+        if m is not None:
+            return tuple(m.groups())
+
+        # Unknown format... take the first two words
+        l = firstline.strip().split()
+        if l:
+            version = l[0]
+            if len(l) > 1:
+                id = l[1]
+        return '', version, id
+
+    _distributor_id_file_re = re.compile("(?:DISTRIB_ID\s*=)\s*(.*)", re.I)
+    _release_file_re = re.compile("(?:DISTRIB_RELEASE\s*=)\s*(.*)", re.I)
+    _codename_file_re = re.compile("(?:DISTRIB_CODENAME\s*=)\s*(.*)", re.I)
+
+    def linux_distribution(distname='', version='', id='',
+                           supported_dists=_supported_dists,
+                           full_distribution_name=1):
+        return _linux_distribution(distname, version, id, supported_dists,
+                                   full_distribution_name)
+
+    def _linux_distribution(distname, version, id, supported_dists,
+                            full_distribution_name):
+
+        """ Tries to determine the name of the Linux OS distribution name.
+            The function first looks for a distribution release file in
+            /etc and then reverts to _dist_try_harder() in case no
+            suitable files are found.
+            supported_dists may be given to define the set of Linux
+            distributions to look for. It defaults to a list of currently
+            supported Linux distributions identified by their release file
+            name.
+            If full_distribution_name is true (default), the full
+            distribution read from the OS is returned. Otherwise the short
+            name taken from supported_dists is used.
+            Returns a tuple (distname, version, id) which default to the
+            args given as parameters.
+        """
+        # check for the Debian/Ubuntu /etc/lsb-release file first, needed so
+        # that the distribution doesn't get identified as Debian.
+        # https://bugs.python.org/issue9514
+        try:
+            with open("/etc/lsb-release", "r") as etclsbrel:
+                for line in etclsbrel:
+                    m = _distributor_id_file_re.search(line)
+                    if m:
+                        _u_distname = m.group(1).strip()
+                    m = _release_file_re.search(line)
+                    if m:
+                        _u_version = m.group(1).strip()
+                    m = _codename_file_re.search(line)
+                    if m:
+                        _u_id = m.group(1).strip()
+                if _u_distname and _u_version:
+                    return (_u_distname, _u_version, _u_id)
+        except (EnvironmentError, UnboundLocalError):
+                pass
+
+        try:
+            etc = os.listdir(_UNIXCONFDIR)
+        except OSError:
+            # Probably not a Unix system
+            return distname, version, id
+        etc.sort()
+        for file in etc:
+            m = _release_filename.match(file)
+            if m is not None:
+                _distname, dummy = m.groups()
+                if _distname in supported_dists:
+                    distname = _distname
+                    break
+        else:
+            return _dist_try_harder(distname, version, id)
+
+        # Read the first line
+        with open(os.path.join(_UNIXCONFDIR, file), 'r',
+                  encoding='utf-8', errors='surrogateescape') as f:
+            firstline = f.readline()
+        _distname, _version, _id = _parse_release_file(firstline)
+
+        if _distname and full_distribution_name:
+            distname = _distname
+        if _version:
+            version = _version
+        if _id:
+            id = _id
+        return distname, version, id
