@@ -11,7 +11,7 @@
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
 import logging
-import sys
+
 from botocore.compat import json
 
 from botocore.utils import set_value_from_jmespath
@@ -91,9 +91,9 @@ class JSONFormatter(FullyBufferedFormatter):
         # the response will be an empty string.  We don't want to print
         # that out to the user but other "falsey" values like an empty
         # dictionary should be printed.
-        if response:
+        if response != {}:
             json.dump(response, stream, indent=4, default=json_encoder,
-                      ensure_ascii=False)
+                    ensure_ascii=False)
             stream.write('\n')
 
 
@@ -231,8 +231,12 @@ class TextFormatter(Formatter):
         try:
             if is_response_paginated(response):
                 result_keys = response.result_keys
-                for page in response:
-                    current = {}
+                for i, page in enumerate(response):
+                    if i > 0:
+                        current = {}
+                    else:
+                        current = response.non_aggregate_part
+
                     for result_key in result_keys:
                         data = result_key.search(page)
                         set_value_from_jmespath(
